import { FetchOptions } from 'ohmyfetch';

interface ResponseMap {
    blob: Blob;
    text: string;
    arrayBuffer: ArrayBuffer;
}
declare type ResponseType = keyof ResponseMap | "json";
declare type MappedType<R extends ResponseType, JsonType = any> = R extends keyof ResponseMap ? ResponseMap[R] : JsonType;
declare type ApiFetchHandler = <T = any, R extends ResponseType = "json">(data?: Record<string, any>, opts?: FetchOptions<R>) => Promise<MappedType<R, T>>;
declare type ApiBuilder = {
    [K: string]: ApiBuilder;
    (...segmentsOrIds: (string | number)[]): ApiBuilder;
} & {
    get: ApiFetchHandler;
    post: ApiFetchHandler;
    put: ApiFetchHandler;
    delete: ApiFetchHandler;
    patch: ApiFetchHandler;
};

/**
 * Minimal, type-safe REST client using JS proxies
 */
declare function createApi<R extends ResponseType = "json">(url: string, defaults?: FetchOptions<R>): ApiBuilder;

export { ApiBuilder, createApi };
